﻿
// ReSharper disable PossiblyUnassignedProperty
// ReSharper disable InconsistentNaming
var Emitter = require("events").EventEmitter;
var Connector = new Emitter();
var fs = require("fs");
var path_module = require("path");
// ReSharper restore InconsistentNaming
// ReSharper restore PossiblyUnassignedProperty

var Conftool;
try {
    var ConftoolPath = "";
    console.log("Environment Variables: " +
        process.env.ARTDAQ_DATABASE_LIB +
        ", " +
        process.env.ARTDAQ_DATABASE_DIR +
        ", " +
        process.env.MRB_BUILDDIR);
    if (!process.env.ARTDAQ_DATABASE_LIB && process.env.ARTDAQ_DATABASE_DIR && process.env.MRB_BUILDDIR) {
        ConftoolPath = path_module.join(process.env["MRB_BUILDDIR"], "artdaq_database", "lib", "conftoolg");
    } else {
        ConftoolPath = path_module.join(process.env["ARTDAQ_DATABASE_LIB"], "node_modules", "conftoolg");
    }
    console.log("Looking for conftoolg module in " + ConftoolPath);
    Conftool = require(ConftoolPath);
    console.log("Module loaded.");
} catch (E) {
    console.log("Error loading conftoolg module...have you setup the artdaq_database product?");
    console.log(E);
}

Connector.Config = {
    dbprovider: "filesystem"
    , configNameFilter: "*"
    , baseDir: process.env.ARTDAQ_DATABASE_DATADIR ? process.env["ARTDAQ_DATABASE_DATADIR"] : path_module.join(process.env["HOME"], "databases")
    , mongoConfig: {
        dbUser: "anonymous"
        , dbHost: "localhost"
        , dbPort: 27017
    }
    , instanceName: "test_db"
};

function setupURI() {
    if (Connector.Config.dbprovider === "filesystem") {
        process.env["ARTDAQ_DATABASE_URI"] = "filesystemdb://" + Connector.Config.baseDir + "/filesystemdb/" + Connector.Config.instanceName;
    } else if (Connector.Config.dbprovider === "mongo") {
        var passStr = "";
        if (fs.existsSync(".dbpasswd")) {
            passStr = ":" + fs.readFileSync(".dbpasswd");
        } else {
            console.log("No Password file (\".dbpasswd\") found. If auth fails, please place this file in the same directory as serverbase.js");
        }
        process.env["ARTDAQ_DATABASE_URI"] = "mongodb://" + Connector.Config.mongoConfig.dbUser + passStr + "@" + Connector.mongoConfig.dbHost + ":" + Connector.mongoConfig.dbPort + "/" + Connector.Config.instanceName;
    } else {
        console.log("Connector.SetConfig: ERROR: Unrecognized database provider");
    }
    console.log("ARTDAQ_DATABASE_URI is " + process.env["ARTDAQ_DATABASE_URI"]);
}

/**
 * Runs a findconfigs query against the database
 * @returns {Object} JSON object returned by the database
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunGetConfigsQuery = function() {
    console.log("Running findconfigs query");
    setupURI();
    var query = {
        filter: {},
        operation: "findconfigs",
        dataformat: "gui"
    };
    if (Connector.Config.configNameFilter.length === 0) {
        Connector.Config.configNameFilter = "*";
    }
    if (Connector.Config.dbprovider === "mongo") {
        query.filter = {
            "configurations.name": {
                "$regex": Connector.Config.configNameFilter
            }
        };
    } else {
        query.filter = {
            "configurations.name": Connector.Config.configNameFilter
        };
    }

    var output = Conftool.find_global_configurations_ui(JSON.stringify(query));
    console.log("findconfigs output: " + JSON.stringify(output));
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
    // console.log(retjson);
    return JSON.parse(output.second);
};

/**
 * Finds the entities currently present in the database
 * @returns {Object} JSON object returned by the database 
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunGetEntitiesQuery = function() {
    console.log("Running findentities query");
    setupURI();
    var query = {
        filter: {},
        operation: "findentities",
        dataformat: "gui"
    };

    var output = Conftool.find_configuration_entities_ui(JSON.stringify(query));
    console.log("Findentities output: " + JSON.stringify(output));
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
    return JSON.parse(output.second);
};

/**
 * Runs the findversions query using the given entity query
 * @param {Object} query - The query to run (as returned by buildfilter)
 * @returns {Object} JSON object returned by the database
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunGetVersionsQuery = function(query) {
    console.log("Running findversions query");
    setupURI();

    var output = Conftool.find_configuration_versions_ui(JSON.stringify(query));
    //console.log("findversions output: " + JSON.stringify(output));
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
    return JSON.parse(output.second);
};

/**
 * Runs a buildfilter operation against the database
 * @param {string} configName - Name of the configuration
 * @param {Object} queryIn - Optional query object
 * @returns {Object} JSON object returned by the database
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunBuildFilterQuery = function (configName, queryIn) {
    setupURI();
    var query = {};

    if (queryIn === undefined) {
        console.log("Getting query for " + configName);
        var configs = Connector.RunGetConfigsQuery().search;
        for (var conf in configs) {
            if (configs.hasOwnProperty(conf)) {
                var config = configs[conf];
                if (config.name === configName) {
                    query = config.query;
                }
            }
        }
    } else {
        query = queryIn;
    }

    console.log("Running buildfilter query: " + JSON.stringify(query));
    var output = Conftool.build_global_configuration_search_filter_ui(JSON.stringify(query));

    for (var file in output.search) {
        if (output.search.hasOwnProperty(file) && query !== null && query !== undefined) {
            if (output.search[file].query.filter["configurations.name"] !== query.filter["configurations.name"]) {
                output.search.splice(file, 1);
            }
        }
    }

    //console.log("buildfilter output: " + JSON.stringify(output));
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
    console.log("BuildfilterQuery return status " + output.first);
    return JSON.parse(output.second);
};

/**
 * Creates a new configuration containing the given file versions
 * @returns {Object} JSON object returned by the database
 * @param {Object} query - Query to run
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunNewConfigQuery = function(query) {
    console.log("Running newconfig query: " + JSON.stringify(query));
    setupURI();

    var output = Conftool.create_new_global_configuration_ui(JSON.stringify(query));
    console.log("newconfig output: " + JSON.stringify(query));

    console.log("RunNewConfigQuery return status " + output.first);
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
};

/**
 * Adds a configuration file to a configuration
 * @param {string} configName - Name of the configuration to modify
 * @param {string} version - Version of the new config file
 * @param {string} collectionName - Collection of new config file
 * @param {Object} entity - Entity information of new config file
 * @param {string} entity.name - Name of the configurable entity
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunAddConfigQuery = function(configName, version, collectionName, entity) {
    console.log("RunAddConfigQuery: configName: " + configName + ", version: " + version + ", collectionName: " + collectionName + ", entity: " + JSON.stringify(entity));
    setupURI();
    var query = {
        filter: {
            "configurable_entity.name": entity.name,
            "version": version
        },
        collection: collectionName,
        configuration: configName,
        operation: "addconfig",
        dataformat: "gui"
    };

    var output = Conftool.add_configuration_to_global_configuration_ui(JSON.stringify(query));
    //console.log("addconfig output: " + JSON.stringify(output));

    console.log("RunAddConfigQuery return status " + output.first);
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
};

/**
 * Runs a store query against the database
 * @param {string} data - File to store
 * @param {string} collectionName - Name of the collection
 * @param {string} version - Version identifier
 * @param {string} entity - Configurable Entity name
 * @param {string} type - Type of data ("gui", "fhicl", etc.)
 * @param {string} configName - Name of the configuration to store data in (Optional)
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunStoreQuery = function(data, collectionName, version, entity, type, configName) {

    console.log("RunStoreConfigQuery: configName: " + configName + ", collectionName: " + collectionName + ", version: " + version + ", entity: " + entity + ", dataType: " + type);
    setupURI();
    var query = {
        version: "" + version,
        collection: "" + collectionName,
        configurable_entity: entity,
        operation: "store",
        dataformat: "" + type
    };
    if (configName !== undefined) {
        query.configuration = "" + configName;
    }

    console.log("RunStoreQuery: Query: " + JSON.stringify(query) + ", Data: " + data);
    var output = Conftool.store_configuration_ui(JSON.stringify(query), data);

    console.log("RunStoreConfigQuery return status " + output.first);
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }
    console.log("RunStoreConfigQuery return value " + output.second);
};

/**
 * Loads a configuration file from the database
 * @param {Object} query - The query to run (as returned by buildfilter)
 * @param {Boolean} noparse - If set, will not run JSON.parse on output (for FHiCL)
 * @returns {Object} Object from database
 * @throws DBOperationFailedException: More information in ex.message
 */
Connector.RunLoadQuery = function(query) {
    console.log("Running load query");
    setupURI();

    var output = Conftool.load_configuration_ui(JSON.stringify(query));
    console.log("Conftool Load output: " + output.first);
    if (!output.first) {
        throw { name: "DBOperationFailedException", message: output.second };
    }

    return JSON.parse(output.second);
};

module.exports = Connector;